/*******************************************************************************
 * Copyright (C) 2017 Philipp Spohr
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
package de.hhu.ba.yoshikoWrapper.gui;

import static javax.swing.GroupLayout.DEFAULT_SIZE;
import static javax.swing.GroupLayout.PREFERRED_SIZE;

import java.awt.Component;
import java.awt.Dialog.ModalExclusionType;
import java.awt.Dialog.ModalityType;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Arrays;

import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.cytoscape.application.swing.CytoPanelComponent;
import org.cytoscape.application.swing.CytoPanelName;
import org.cytoscape.util.swing.BasicCollapsiblePanel;
import org.cytoscape.work.AbstractTask;
import org.cytoscape.work.TaskIterator;

import de.hhu.ba.yoshikoWrapper.core.AlgorithmTask;
import de.hhu.ba.yoshikoWrapper.core.CyCore;
import de.hhu.ba.yoshikoWrapper.core.GraphicsLoader;
import de.hhu.ba.yoshikoWrapper.core.Hint;
import de.hhu.ba.yoshikoWrapper.core.LocalizationManager;
import de.hhu.ba.yoshikoWrapper.core.ParameterSet;
import de.hhu.ba.yoshikoWrapper.core.YoshikoLoader;

/**This class describes the Swing Panel that the user interacts with in cytoscape
 *
 */
@SuppressWarnings("serial")
public class MainPanel extends JPanel implements CytoPanelComponent {
	
	//SWING COMPONENTS
		
	private final YoshikoHeader header;
	
	private final JCheckBox showAdvancedOptions;
	private final ArrayList<JComponent> advancedOptions;

	//Those two work with factories whysoever
	private final BasicCollapsiblePanel langPanel;
	private final BasicCollapsiblePanel libraryPanel;
	
	//Those don't work really as collapsible panels and cause glitches which forces the workaround of wrapping them
	//Probably someone used getClass() == instead of instanceof so extending BasicCollapsiblePanel is sadly not possible
	private final BasicCollapsiblePanel ecPanelWrapper;
	private final EditCostPanel ecPanel;
	
	private final BasicCollapsiblePanel reductionWrapper;
	private final ReductionRulesChooser reductionRulesChooser;
	
	private final BasicCollapsiblePanel opWrapper;
	private final OperationModePanel opModePanel;	
	
	/**
	 * Main constructor, creates a new Panel and initializes subcomponents
	 */
	public MainPanel() {

		//Initialize Swing components		
		header = new YoshikoHeader();
		
		showAdvancedOptions = new JCheckBox(LocalizationManager.get("showAdvanced"));
		showAdvancedOptions.addActionListener(toggleAdvancedOptionsListener);
		
		langPanel = LanguageSwitcherPanelFactory.createLanguageSwitcherPanel();
		
		libraryPanel = LibraryPanelFactory.createLibraryPanel();
		//If no library is loaded yet the obvious panel should be showing up
		if (!YoshikoLoader.isLibraryLoaded()) {
			libraryPanel.setCollapsed(false);
		}
		
		ecPanel = new EditCostPanel();
		ecPanelWrapper = new BasicCollapsiblePanel(LocalizationManager.get("editingCostPanel"));
		ecPanelWrapper.add(ecPanel);
		ecPanelWrapper.setCollapsed(false);
		
		opModePanel = new OperationModePanel();
		opWrapper = new BasicCollapsiblePanel(LocalizationManager.get("operationMode"));
		opWrapper.add(opModePanel);
		
		reductionWrapper = new BasicCollapsiblePanel(LocalizationManager.get("redRuleChooserTitle"));
		reductionRulesChooser = new ReductionRulesChooser();
		reductionWrapper.add(reductionRulesChooser);
		
		JButton runButton = new JButton(LocalizationManager.get("run"));
		runButton.addActionListener(buttonListener);

		//Add components to main panel
		SwingUtil.addAll(this,
				header,
				showAdvancedOptions,
				langPanel,
				libraryPanel,
				ecPanelWrapper,
				reductionWrapper,
				opWrapper,
				runButton
		);
		
		//Manage all advanced components separately to enable toggling
		advancedOptions = new ArrayList<JComponent>();
		advancedOptions.addAll(
				Arrays.asList(
						opWrapper,
						reductionWrapper
						)
		);
		
		showAdvancedOptions(false);
		
		//Layout
		GroupLayout layout = new GroupLayout(this);
		layout.setAutoCreateGaps(true);
		layout.setAutoCreateContainerGaps(true);
		layout.setHorizontalGroup(layout.createParallelGroup(Alignment.LEADING,true)
				.addComponent(header,DEFAULT_SIZE, PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(showAdvancedOptions,DEFAULT_SIZE, PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(langPanel,DEFAULT_SIZE, PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(libraryPanel,DEFAULT_SIZE, PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(ecPanelWrapper,DEFAULT_SIZE, PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(reductionWrapper,DEFAULT_SIZE, PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(opWrapper,DEFAULT_SIZE, PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(runButton,DEFAULT_SIZE, PREFERRED_SIZE,PREFERRED_SIZE)
		);
		
		layout.setVerticalGroup(layout.createSequentialGroup()
				.addComponent(header,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(showAdvancedOptions,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(langPanel,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(libraryPanel,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(ecPanelWrapper,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(reductionWrapper,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(opWrapper,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(runButton,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)	
		);
		
		this.setLayout(layout);
				
	}
	
	private ActionListener toggleAdvancedOptionsListener = new ActionListener() {

		@Override
		public void actionPerformed(ActionEvent e) {
			showAdvancedOptions(showAdvancedOptions.isSelected());
		}
		
	};
	
	private void showAdvancedOptions(boolean show) {
		for (JComponent j : advancedOptions) {
			j.setVisible(show);
		}
		setPreferredSize(getPreferredSize()); // <<< Swing at its finest
		revalidate();
	}

	/**
	 * ButtonListener for the "Run" Button
	 * Handles calling the algorithm and fetching/passing the arguments from swing components
	 */
	private ActionListener buttonListener = new ActionListener() {

		private final TaskIterator iterator = new TaskIterator();
		@Override
		public void actionPerformed(ActionEvent e) {
			
			//SWING BLACK MAGIC
			Window noWindow = null;
			JDialog statusWindow = new JDialog(noWindow);
			JDialog popupLevel = new JDialog(statusWindow);
			popupLevel.setAlwaysOnTop(true);
			popupLevel.setModalityType(ModalityType.APPLICATION_MODAL);
			popupLevel.setModalExclusionType(ModalExclusionType.APPLICATION_EXCLUDE);
			statusWindow.setModalityType(ModalityType.MODELESS);
			statusWindow.setAlwaysOnTop(false);
			//SWING BLACK MAGIC
			
			if (YoshikoLoader.isLibraryLoaded()){
				
				//User has mapped no values -> Annoy him
				if (ecPanel.getWeightColumn() == null) {
					HintManager.showMappingHint(Hint.USE_MAPPING);
				}
				
				AbstractTask yoshiko = new AlgorithmTask(
					popupLevel,
					CyCore.cy.getCurrentNetwork(),
					fetchParameter()
				);
				/**
				 * Don't queue multiple runs at once because that will get messy
				 */
				if (!iterator.hasNext()) {
					iterator.append(yoshiko);
					
					CyCore.dialogTaskManager.setExecutionContext(statusWindow);
					CyCore.dialogTaskManager.execute(iterator);
					
				}
			}
			else {
				JOptionPane.showMessageDialog(	
					null,
					LocalizationManager.get("noLibTitle"),
					LocalizationManager.get("noLibMessage"),
					JOptionPane.ERROR_MESSAGE
				);
			}
		}
	};
	
	/**
	 * Fetches all the parameters from the various swing components and packs them into a neat abstract wrapper class
	 * @return The currently selected parameter wrapped in a ParameterSet
	 */
	private ParameterSet fetchParameter() {
		ParameterSet ret = new ParameterSet();
		ret.timeLimit = opModePanel.getTimeLimit();
		ret.weightColumn = ecPanel.getWeightColumn();
		ret.permanentColumn = ecPanel.getPermanentColumn();
		ret.forbiddenColumn = ecPanel.getForbiddenColumn();
		ret.defaultInsertionCost = ecPanel.getDefaultInsertionCost();
		ret.defaultDeletionCost = ecPanel.getDefaultDeletionCost();
		ret.reductionRulesBitMask = reductionRulesChooser.getBitMask();
		ret.snrMultFactor = reductionRulesChooser.getMultFactor();
		ret.useTriangleCuts = opModePanel.useTriangleCuts();
		ret.usePartitionCuts = opModePanel.usePartitionCuts(); 
		ret.useHeuristic = opModePanel.useHeuristic();
		ret.solCount = opModePanel.getSolCount();
		ret.disableMultiThreading = opModePanel.isMultiThreadingDisabled();
		return ret;
	}

	
	//GETTER / SETTER
	
	public ColumnMapper getColumnMapper() {
		return ecPanel.getColumnMapper();
	}

	public Component getComponent() {
		return this;
	}

	/* (non-Javadoc)
	 * @see org.cytoscape.application.swing.CytoPanelComponent#getCytoPanelName()
	 */
	public CytoPanelName getCytoPanelName() {
		//By convention most plugins that provide a "toolbox"-like interface use the WEST orientation
		return CytoPanelName.WEST;
	}

	public String getTitle() {
		return "Yoshiko";
	}
	
	public Icon getIcon() {
		return GraphicsLoader.getLogo(16);
	}
}
